SystemOrganization addCategory: #Seamless!

TestCase subclass: #SeamlessSimpleSocketStreamTransporterTest
	instanceVariableNames: 'peer1 peer2 initialData receivedData numberOfSends'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessSimpleSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 21:11'!
setUp

	initialData := (1 to: 100000) asOrderedCollection asString asByteArray.
	
	receivedData := OrderedCollection new.
		
	numberOfSends := 10.

	peer1 := (SeamlessSocketStreamTransporter new startOn: 8080) onDataReceivedEvaluate: [:conId :data |
		receivedData add: data.
	].

	peer2 := (SeamlessSocketStreamTransporter new startOn: 8081) onDataReceivedEvaluate: [:conId :data |
		(1 to: numberOfSends) do: [:i |
			peer2 send: data through: (peer2 connectionIds at: 1).
		]
	].
	
	peer1 connectOn: '127.0.0.1:8081'.

	peer2 connectOn: '127.0.0.1:8080'.  
	
! !

!SeamlessSimpleSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 15:23'!
tearDown

	peer1 stop.
	peer2 stop.	
! !

!SeamlessSimpleSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 16:08'!
testEcho

	(1 to: numberOfSends) do: [: i |
		peer1 send: initialData through: (peer1 connectionIds at: 1).
	].

	(Delay forSeconds: 20) wait.
	
	self assert: ((receivedData size) = (numberOfSends raisedTo: 2)).
	
	receivedData do: [:data |
		self assert: (data = initialData).
	].
! !

TestCase subclass: #SeamlessSocketStreamTransporterTest
	instanceVariableNames: 'peers initialData receivedData numberOfPeers stoppedConnections numberOfSends simulationWaitingTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

SeamlessSocketStreamTransporterTest class
	instanceVariableNames: 'transporterClass'!

SeamlessSocketStreamTransporterTest class
	instanceVariableNames: 'transporterClass'!

!SeamlessSocketStreamTransporterTest class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 13:44'!
initialize

	super initialize.
	self transporterClass: SeamlessSocketStreamTransporter! !

!SeamlessSocketStreamTransporterTest class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 13:37'!
transporterClass
	^ transporterClass! !

!SeamlessSocketStreamTransporterTest class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 13:37'!
transporterClass: anObject
	transporterClass := anObject! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 20:48'!
connectAll

	peers doWithIndex: [:peer :index |
		((index + 1) to: numberOfPeers) do: [:peerIndex |
				peer connectOn: (peers at: peerIndex) serverAddress.
		]
	].! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 22:07'!
setUp

	initialData := (1 to: 1000000) asOrderedCollection asString asByteArray.
	receivedData := OrderedCollection new.
	peers := OrderedCollection new.
	stoppedConnections := OrderedCollection new.
	numberOfPeers := 20. "numberOfPeers Transporters, almost numberOfPeers ^ 2 -- sockets and concurrent connections"
	numberOfSends := 20.
	
	(1 to: numberOfPeers) do: [:i |
		peers add: self class transporterClass new.
		receivedData add: OrderedCollection new.	
	]
	! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 20:48'!
startAll
	
	peers doWithIndex: [:peer :index |
			peer startOn: 8080 + index.
	].! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:30'!
stopAll

	peers do: [:peer | peer stop]! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:30'!
tearDown

	self stopAll! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:30'!
testConnectOn

	self startAll connectAll.
	
	peers do: [:peer |
		self assert: (peer connectionIds size = (numberOfPeers - 1)).	
	]! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 21:35'!
testOnDataReceivedEvaluate

	self startAll connectAll.
	
	peers doWithIndex: [:peer :index |
		peer onDataReceivedEvaluate: [:connectionId :data |
			 (receivedData at: index) add: data. 
		]	
	].

	peers doWithIndex: [:peer :index |
		
		[
		
			(peer connectionIds) do: [:connectionId | 	
	
					(1 to: numberOfSends) do: [:i |
					
						[peer send: initialData through: connectionId. Processor activeProcess terminate. ] fork. "multiple threads simulation ?"	
								
					].

			].
		
			Processor activeProcess terminate.
		
		] fork."multiple peers simulation ?"
	
	].

	self wait.
	
	receivedData do: [:dataCollection |
		self assert: (dataCollection size = (numberOfSends*(peers size -1))).
		dataCollection do: [:data |
			self assert: (data = initialData).
		].
	].	
! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 16:01'!
testOnNewConnectionEvaluate

	|connectionIds|
	
	connectionIds := OrderedCollection new.
		
	self startAll.
	
	peers doWithIndex: [:peer :index |
		connectionIds add: OrderedCollection new.
		peer onNewConnectionEvaluate: [:connectionId | (connectionIds at: index) add: connectionId].
	].
	
	peers doWithIndex: [:peer :index |
		peers doWithIndex: [:otherPeer :otherIndex |
			index < otherIndex ifTrue: [
				peer connectOn: otherPeer serverAddress.
			]
		]	
	].

	connectionIds do: [:connectionsPerPeer |
		self assert: (connectionsPerPeer size = (peers size - 1))		
	 ]! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 14:25'!
testStartOn

	self startAll! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 13:08'!
testStop
	
	self startAll connectAll.
	
	peers doWithIndex: [:peer :index |
		stoppedConnections add: OrderedCollection new.
		peer onStoppedEvaluate: [:conId |
			(stoppedConnections at: index) add: conId.
		]	
	].

	self stopAll.

	peers doWithIndex: [:peer :index |
		self assert: ((stoppedConnections at: index) size = (numberOfPeers - 1)).
	].! !

!SeamlessSocketStreamTransporterTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 17:16'!
wait

	(Delay forSeconds: numberOfPeers*numberOfSends) wait.! !

SeamlessSocketStreamTransporterTest subclass: #SeamlessZodiacTransporterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessZodiacTransporterTest class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:45'!
initialize 

	super initialize.
	self transporterClass: SeamlessZodiacTransporter ! !

TestCase subclass: #SeamlessZodiacConnectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:33'!
setUp! !

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:34'!
tearDown! !

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 12:02'!
testReceiveData! !

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:36'!
testSendData! !

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:36'!
testStart! !

!SeamlessZodiacConnectionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:36'!
testStop! !

Object subclass: #AbstractSeamlessConnection
	instanceVariableNames: 'id dataAvailableCallback localAddress remoteAddress incomingDataProcess stopCallback sendSemaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

AbstractSeamlessConnection class
	instanceVariableNames: 'nextId'!

AbstractSeamlessConnection class
	instanceVariableNames: 'nextId'!

!AbstractSeamlessConnection class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 15:17'!
initialize
	super initialize.
	nextId := 0! !

!AbstractSeamlessConnection class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 14:53'!
new

	nextId := nextId + 1.
	^ super new id: nextId.! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:06'!
cleanUp 

	incomingDataProcess terminate! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 21:16'!
dataReceived: data

	"as the callback is currently implemented -- the block evaluation is atomic i.e it has to return for the next receive to continue"

	dataAvailableCallback ifNotNil: [
		dataAvailableCallback value: self id value: data.
	]
	! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 14:52'!
id
	^ id! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 14:52'!
id: anObject
	id := anObject! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 20:49'!
initialize 

	sendSemaphore := Semaphore forMutualExclusion ! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:13'!
isConnected

	self subclassResponsibility ! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 15:28'!
localAddress
	^ localAddress! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 15:28'!
localAddress: anObject
	localAddress := anObject! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 10:57'!
onDataReceivedEvaluate: aBlock

	dataAvailableCallback := aBlock.! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 13:59'!
onStopped

	stopCallback ifNotNil: [
		stopCallback value: self id.
		stopCallback := nil. "normally only one side initiates stop -- we nil the block to avoid duplicated callbacks"
	].

	self cleanUp.
	! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:05'!
onStoppedEvaluate: aBlock

	stopCallback := aBlock.! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:09'!
receiveData

	|header|
	
	header := self receiveInteger.
	
	(header = -1) ifTrue: [
		self onStopped.
	] ifFalse: [
		self dataReceived: (self receivePacketSized: header)
	]
	
	! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:53'!
receiveInteger

	self subclassResponsibility ! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:25'!
receivePacketSized: packetSize

	self subclassResponsibility ! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 15:44'!
remoteAddress
	^ remoteAddress! !

!AbstractSeamlessConnection methodsFor: 'accessing' stamp: 'NickPapoulias 5/9/2012 15:44'!
remoteAddress: anObject
	remoteAddress := anObject! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 20:50'!
sendData: dataAsByteArray

	sendSemaphore critical: [

		self sendPacketSize: dataAsByteArray size.
		self sendPacket: dataAsByteArray.
		
	].
		! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:53'!
sendInteger: int32

	self subclassResponsibility ! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 22:10'!
sendPacket: data

	self subclassResponsibility ! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:22'!
sendPacketSize: int32

	self sendInteger: int32.! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 16:29'!
sendStop

		self sendInteger: -1
! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:47'!
start

	incomingDataProcess := [
		
		[self receiveData] repeat
		
	] fork.! !

!AbstractSeamlessConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:50'!
stop

	self isConnected ifTrue: [
		self sendStop.
	].

	self onStopped.! !

AbstractSeamlessConnection subclass: #SeamlessSocketStreamConnection
	instanceVariableNames: 'stream socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

SeamlessSocketStreamConnection class
	instanceVariableNames: 'SocketStreamClass'!

SeamlessSocketStreamConnection class
	instanceVariableNames: 'SocketStreamClass'!

!SeamlessSocketStreamConnection class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 12:09'!
initialize

	super initialize.
	self socketStreamClass: SocketStream.! !

!SeamlessSocketStreamConnection class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 12:19'!
newOnSocket: aSocket 

	^ self new
		withSocket: aSocket;
		localAddress: aSocket localAddress asString , ':' , aSocket localPort asString;
		remoteAddress: aSocket remoteAddress asString , ':' , aSocket remotePort asString.
! !

!SeamlessSocketStreamConnection class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:17'!
openConnectionToHost: address port: port
	
	|stream|
	
	stream := self socketStreamClass openConnectionToHost: address port: port. 
	
	^ self new
		withStream: stream;
		localAddress: stream socket localAddress asString , ':' , stream socket localPort asString;
		remoteAddress: stream socket remoteAddress asString , ':' , stream socket remotePort asString.
! !

!SeamlessSocketStreamConnection class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 12:08'!
socketStreamClass
	^ SocketStreamClass! !

!SeamlessSocketStreamConnection class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 12:08'!
socketStreamClass: anObject
	SocketStreamClass := anObject! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:15'!
isConnected 

	^ stream isConnected ! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:54'!
receiveInteger
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + stream next asInteger].
	(s bitAnd: 16r80000000) = 0
		ifTrue: [^ s]
		ifFalse: [^ -1 - s bitInvert32]! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 15:25'!
receivePacketSized: packetSize

	^ (stream next: packetSize) asByteArray.! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:17'!
sendInteger: int32
	"Write a signed integer to the next 4 bytes"
	| pos |
	pos := int32 < 0
		ifTrue: [(0-int32) bitInvert32 + 1]
		ifFalse: [int32].
	1 to: 4 do: [:i | stream nextPut: (pos digitAt: 5-i)].
	stream flush.
	^ int32! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:20'!
sendPacket: dataAsByteArray

	stream nextPutAll: dataAsByteArray.
	stream flush! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 11:55'!
stop

	super stop.

	stream close.! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:18'!
withSocket: aSocket 

	socket := aSocket.
	stream := self class socketStreamClass on: socket.
	stream timeout: 0.! !

!SeamlessSocketStreamConnection methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:19'!
withStream: aStream 

	socket := aStream socket.
	stream := aStream.
	stream timeout: 0.! !

SeamlessSocketStreamConnection subclass: #SeamlessZodiacConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessZodiacConnection class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:45'!
initialize

	super initialize.
	self socketStreamClass: ZdcSocketStream! !

AbstractSeamlessConnection subclass: #SeamlessZincCommetConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

Object subclass: #SeamlessAbstractTransporter
	instanceVariableNames: 'incomingConnectionsProcess seamlessConnections dataAvailableCallback newConnectionCallback stoppedCallback serverAddress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

SeamlessAbstractTransporter class
	instanceVariableNames: 'seamlessConnectionClass'!

SeamlessAbstractTransporter class
	instanceVariableNames: 'seamlessConnectionClass'!

!SeamlessAbstractTransporter class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 12:59'!
seamlessConnectionClass
	^ seamlessConnectionClass! !

!SeamlessAbstractTransporter class methodsFor: 'accessing' stamp: 'NickPapoulias 5/10/2012 12:59'!
seamlessConnectionClass: anObject
	seamlessConnectionClass := anObject! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 15:38'!
addNewConnection: aSeamlessConnection

	seamlessConnections at: aSeamlessConnection id put: aSeamlessConnection.
	self newConnectionAdded: aSeamlessConnection.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 15:40'!
connectOn: aRemoteAddressAsString

	self subclassResponsibility.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 15:50'!
connectionIds

	^ seamlessConnections keys.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:23'!
handleConnectionRequests

	self subclassResponsibility ! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 14:07'!
initialize 

	super initialize.
	seamlessConnections := Dictionary new.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 15:39'!
newConnectionAdded: aSeamlessConnection

	newConnectionCallback ifNotNil: [newConnectionCallback value: aSeamlessConnection id].! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 09:28'!
onDataReceivedEvaluate: aBlock

	dataAvailableCallback := aBlock.
	seamlessConnections do: [:sConnection |
		sConnection onDataReceivedEvaluate: aBlock.	
	]! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/6/2012 16:12'!
onNewConnectionEvaluate: aBlock

	newConnectionCallback := aBlock.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:43'!
onStoppedEvaluate: aBlock

	stoppedCallback := aBlock.
	seamlessConnections do: [:sConnection |
		sConnection onStoppedEvaluate:  aBlock.	
	]! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 16:00'!
send: dataAsByteArray through: aSeamlessConnectionId

	(seamlessConnections at: aSeamlessConnectionId) sendData: dataAsByteArray.! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 08:49'!
serverAddress

	^ serverAddress! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 08:50'!
serverAddress: aServerIdentifier

	serverAddress := aServerIdentifier ! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 20:09'!
start

	incomingConnectionsProcess := [ 
	
			      self handleConnectionRequests.
	
	] fork! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 15:35'!
startOn: anIdentifier

	self subclassResponsibility ! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 15:01'!
stop

	seamlessConnections ifNotNil: [
		seamlessConnections keysDo: [:aConnectionId | 
			self stop: aConnectionId.	
		]
	].

	incomingConnectionsProcess ifNotNilDo: [:process | process terminate ].
	! !

!SeamlessAbstractTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/9/2012 14:59'!
stop: aSeamlessConnectionId

	(seamlessConnections at: aSeamlessConnectionId) stop.! !

SeamlessAbstractTransporter subclass: #SeamlessSocketStreamTransporter
	instanceVariableNames: 'serverSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessSocketStreamTransporter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 12:59'!
initialize

	super initialize.
	self seamlessConnectionClass: SeamlessSocketStreamConnection.! !

!SeamlessSocketStreamTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:20'!
connectOn: aRemoteAddressAsString
	
	|clientStream address port newConnection|

	address := ByteArray new writeStream.
	
	((aRemoteAddressAsString findTokens: #($. $:)) doWithIndex: [:item :index |
		(index < 5) ifTrue: [
			address nextPut: item asInteger		
		] ifFalse: [
			port := item asInteger
		]	
	]).
	
	newConnection := (self class seamlessConnectionClass openConnectionToHost: address contents port: port)
									onDataReceivedEvaluate: dataAvailableCallback;
									start.
	
	self addNewConnection: newConnection.
	
	^ newConnection id.

! !

!SeamlessSocketStreamTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 20:09'!
handleConnectionRequests

	|clientSocket|
	
	[
	
		clientSocket := serverSocket waitForAcceptFor: 10.
		clientSocket ifNotNil: [ 
				| newConnection |
				newConnection := (self class seamlessConnectionClass newOnSocket: clientSocket)
									onDataReceivedEvaluate: dataAvailableCallback;
									start.
				self addNewConnection: newConnection 
		]
	
	] repeat ! !

!SeamlessSocketStreamTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 12:56'!
start

	self startOn: 8080.! !

!SeamlessSocketStreamTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:25'!
startOn: aPortNumber

	| clientSocket stream aPortAsString |

	aPortAsString := aPortNumber asString.

	(serverSocket := Socket newTCP)
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 4096;
		setOption: 'SO_RCVBUF' value: 4096.

	serverSocket listenOn: aPortAsString asNumber backlogSize: 32.

	serverSocket isValid
		ifFalse: [ self error: 'Cannot create socket on port ' , aPortAsString ].
		
	self serverAddress: '127.0.0.1:' , aPortAsString.

	super start! !

!SeamlessSocketStreamTransporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 12:36'!
stop
	
	super stop.
	
	serverSocket close.! !

SeamlessSocketStreamTransporter subclass: #SeamlessZodiacTransporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessZodiacTransporter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:00'!
initialize

	super initialize.
	self seamlessConnectionClass: SeamlessZodiacConnection! !

SeamlessAbstractTransporter subclass: #SeamlessZincCommetTransporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessZincCommetTransporter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2012 13:01'!
initialize

	super initialize.
	self seamlessConnectionClass: SeamlessZincCommetConnection.! !

Object subclass: #SeamlessDependencies
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seamless'!

!SeamlessDependencies class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 14:32'!
load

"Gofer it
	squeaksource: 'ZincHTTPComponents';
	package: 'Zinc-Zodiac';
	load.
	
Gofer it
	squeaksource: 'Zodiac';
	package: 'Zodiac-Core';
	package: 'Zodiac-Tests';
	package: 'Zodiac-Extra';
	load.
	
(ConfigurationOfFuel project version: #bleedingEdge) load

Gofer it
url: 'http://ss3.gemstone.com/ss/Ghost';
package: 'ConfigurationOfGhost';
load.

(Smalltalk at: #ConfigurationOfGhost) load"! !

!SeamlessDependencies class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2012 14:30'!
misc

	VirtualMachine maxExternalSemaphoresSilently: 65535
	VirtualMachine maxExternalSemaphores

	"also remember the limit of concurrent connections from the OS"

	"socket best strategy -- one socket per execution path"! !
SeamlessSocketStreamTransporter initialize!
SeamlessZincCommetTransporter initialize!
AbstractSeamlessConnection initialize!
SeamlessSocketStreamConnection initialize!
SeamlessZodiacConnection initialize!
SeamlessSocketStreamTransporterTest initialize!
SeamlessZodiacTransporterTest initialize!
SeamlessZodiacTransporter initialize!
